import random
from typing import Optional

from pyvisa_mock.base.base_mocker import BaseMocker, scpi


class EspecPr3jMocker(BaseMocker):
    """
    A mocker for an Espec PR-3J environmental chamber.
    """

    LINE_TERMINATION = "\r\n"
    """The line termination character for the mocker."""

    DEFAULT_HUMIDITY = 50.0
    """The default humidity value for the mocker, used when the humidity control is
    disabled."""

    def __init__(
        self,
        call_delay: float = 0.0,
        temperature_steps: int = 10,
        humidity_steps: int = 10,
    ):
        super().__init__(call_delay=call_delay)
        self._target_temperature = 0.0
        self._lower_temperature = 0.0
        self._upper_temperature = 0.0
        self._temperature_steps: list[float] = [0.0]
        self._temperature_num_steps = temperature_steps

        self._target_humidity: Optional[float] = 0.0
        self._lower_humidity = 0.0
        self._upper_humidity = 0.0
        self._humidity_steps: list[float] = [0.0]
        self._humidity_num_steps = humidity_steps

        self._mode = "STANDBY"

    @scpi("TEMP, S<temperature>")
    def _set_target_temperature(self, temperature: float) -> str:
        self._target_temperature = temperature

        # we want to simulate a linear temperature change
        # calculate the step size based on the number of steps and difference
        current = self._current_temperature
        step_jump = temperature - current
        step_size = step_jump / self._temperature_num_steps

        self._temperature_steps = []
        for step in range(self._temperature_num_steps + 1):
            self._temperature_steps.append(current + step * step_size)

        return f"OK:TEMP, S{temperature:.1f}{self.LINE_TERMINATION}"  # noqa E231

    @property
    def _current_temperature(self) -> float:
        # always return the last element of the list
        # pop it from the list, unless it is the last one
        if len(self._temperature_steps) > 1 and self._mode == "CONSTANT":
            return self._temperature_steps.pop(0)
        return self._temperature_steps[0]

    @scpi("TEMP, H<temperature>")
    def _set_upper_temperature(self, temperature: float) -> str:
        self._upper_temperature = temperature
        return f"OK:TEMP, H {temperature:.1f}{self.LINE_TERMINATION}"  # noqa E231

    @scpi("TEMP, L<temperature>")
    def _set_lower_temperature(self, temperature: float) -> str:
        self._lower_temperature = temperature
        return f"OK:TEMP, L {temperature:.1f}{self.LINE_TERMINATION}"  # noqa E231

    @scpi("TEMP?")
    def _get_temperature_status(self) -> str:
        response = f"{self._current_temperature:.1f}"  # noqa E231
        response += f",{self._target_temperature:.1f}"  # noqa E231
        response += f",{self._upper_temperature:.1f}"  # noqa E231
        response += f",{self._lower_temperature:.1f}"  # noqa E231
        response += f"{self.LINE_TERMINATION}"

        return response

    @scpi("HUMI, S<humidity>")
    def _set_target_humidity(self, humidity: str) -> str:
        if type(humidity) == str and humidity == "OFF":
            self._target_humidity = None
            return f"OK:HUMI, S{humidity}{self.LINE_TERMINATION}"  # noqa E231

        self._target_humidity = float(humidity)

        # we want to simulate a linear humidity change
        # calculate the step size based on the number of steps and difference
        current = self._current_humidity
        step_jump = self._target_humidity - current
        step_size = step_jump / self._humidity_num_steps

        self._humidity_steps = []
        for step in range(self._humidity_num_steps + 1):
            self._humidity_steps.append(current + step * step_size)

        return (
            f"OK:HUMI, S{self._target_humidity:.1f}"  # noqa E231
            f"{self.LINE_TERMINATION}"
        )

    @property
    def _current_humidity(self) -> float:
        if self._target_humidity is None:
            return self.DEFAULT_HUMIDITY

        # always return the last element of the list
        # pop it from the list, unless it is the last one
        if len(self._humidity_steps) > 1 and self._mode == "CONSTANT":
            return self._humidity_steps.pop(0)

        return self._humidity_steps[0]

    @scpi("HUMI, H<humidity>")
    def _set_upper_humidity(self, humidity: float) -> str:
        self._upper_humidity = humidity
        return f"OK:HUMI, H{humidity:.0f}\r\n"  # noqa E231

    @scpi("HUMI, L<humidity>")
    def _set_lower_humidity(self, humidity: float) -> str:
        self._lower_humidity = humidity
        return f"OK:HUMI, L{humidity:.0f}{self.LINE_TERMINATION}"  # noqa E231

    @scpi("HUMI?")
    def _get_humidity_status(self) -> str:
        response = f"{self._current_humidity:.0f}"  # noqa E231

        if self._target_humidity is None:
            response += ",OFF"  # noqa E231
        else:
            response += f",{self._target_humidity:.0f}"  # noqa E231

        response += f",{self._upper_humidity:.0f}"  # noqa E231
        response += f",{self._lower_humidity:.0f}"  # noqa E231
        response += f"{self.LINE_TERMINATION}"

        return response

    @scpi("MODE, <mode>")
    def _set_mode(self, mode: str) -> str:
        if mode not in ["STANDBY", "OFF", "CONSTANT", "RUN"]:
            return f"NA:DATA NOT READY{self.LINE_TERMINATION}"  # noqa E231

        self._mode = mode
        return f"OK:MODE, {mode}{self.LINE_TERMINATION}"  # noqa E231

    @scpi("MODE?")
    def _get_mode(self) -> str:
        return self._mode

    @scpi("MON?")
    def _get_monitor(self) -> str:
        response = f"{self._current_temperature:.1f}"  # noqa E231
        response += f",{self._current_humidity:.0f}"  # noqa E231
        response += f",{self._mode}"  # noqa E231
        response += ",0"  # noqa E231 number of alarms occurring
        response += f"{self.LINE_TERMINATION}"

        return response

    @scpi("%?")
    def _get_heaters(self) -> str:
        temperature_heater = random.random() * 100.0
        humidity_heater = random.random() * 100.0

        response = f"2,{temperature_heater},"  # noqa E231
        response += f"{humidity_heater}{self.LINE_TERMINATION}"  # noqa E231

        return response



import pytest
from espec_pr3j_mocker import EspecPr3jMocker
from pyvisa import ResourceManager
from pyvisa_mock.base.register import register_resource

from espec_pr3j import EspecPr3j, OperationMode

RESOURCE_PATH = "MOCK0::mock1::INSTR"


def pytest_addoption(parser):
    parser.addoption(
        "--hil", action="store_true", help="Run tests on hardware-in-the-loop"
    )
    parser.addoption("--hil_hostname", help="Hostname of the device under test")


@pytest.fixture(scope="session")
def hil(request):
    return request.config.option.hil is not None and request.config.option.hil


@pytest.fixture(scope="session")
def hil_hostname(hil, request):
    if not hil:
        return None

    return request.config.option.hil_hostname


@pytest.fixture(scope="module")
def environmental_chamber(hil, hil_hostname):
    if hil:
        chamber = EspecPr3j(
            hostname=hil_hostname,
        )
    else:
        mock_environmental_chamber = EspecPr3jMocker()
        register_resource(RESOURCE_PATH, mock_environmental_chamber)

        resource_manager = ResourceManager(visa_library="@mock")

        chamber = EspecPr3j(
            resource_path=RESOURCE_PATH, resource_manager=resource_manager
        )

    yield chamber

    chamber.set_mode(OperationMode.STANDBY)



from dataclasses import dataclass
from enum import Enum
from typing import Optional


@dataclass
class TemperatureStatus:
    """
    The temperature status of the environmental chamber. All values are in Celsius.
    """

    current_temperature: float
    """The current temperature of the environmental chamber"""

    target_temperature: float
    """The target temperature of the environmental chamber"""

    upper_limit: float
    """The upper temperature limit of the environmental chamber"""

    lower_limit: float
    """The lower temperature limit of the environmental chamber"""


@dataclass
class HumidityStatus:
    """
    The humidity status of the environmental chamber. All values are in percentage.
    """

    current_humidity: float
    """The current humidity of the environmental chamber"""

    target_humidity: Optional[float]
    """The target humidity of the environmental chamber. None if the humidity control is
    disabled."""

    upper_limit: float
    """The upper humidity limit of the environmental chamber"""

    lower_limit: float
    """The lower humidity limit of the environmental chamber"""


@dataclass
class HeatersStatus:
    """
    The status of the heaters. All values are in percentage.
    """

    temperature_heater: float
    """The output of the temperature heater"""

    humidity_heater: float
    """The output of the humidity heater"""


class OperationMode(Enum):
    """
    The operation mode of the environmental chamber.
    """

    OFF = "OFF"
    """The panel is powered off"""

    CONSTANT = "CONSTANT"
    """Constant operation"""

    STANDBY = "STANDBY"
    """All operations are stopped"""

    RUN = "RUN"
    """A program is running"""

    @classmethod
    def from_str(cls, value: str) -> "OperationMode":
        """
        Convert a string to an OperationMode.

        Args:
            value: The string to convert. The string is case-insensitive.
        """
        return cls[value.upper()]

    def __str__(self) -> str:
        return self.value


@dataclass
class TestAreaState:
    """
    The state of the test area.
    """

    current_temperature: float
    """The current temperature of the test area, in Celsius"""

    current_humidity: float
    """The current humidity of the test area, in percentage"""

    operation_state: OperationMode
    """The operation state"""

    number_of_alarms: int
    """The number of alarms occurring"""



#!/usr/bin/python3
import logging
import re
import time
from typing import Optional

import pyvisa

from .data_classes import (
    HeatersStatus,
    HumidityStatus,
    OperationMode,
    TemperatureStatus,
    TestAreaState,
)
from .exceptions import MonitorError, SettingError

_LOGGER = logging.getLogger(__name__)


class EspecPr3j:
    """
    Implements the basic operation of the environmental chamber.

    Args:
        `hostname (Optional[str])`: Host name of the environmental chamber. Default is
            None. If None, the resource_path must be provided. Can't be used with
            resource_path. It can be an IP address.
        `temperature_accuracy (Optional[float])`: The accuracy considered when setting
            the temperature. Default is 0.5.
        `humidity_accuracy (Optional[float])`: The accuracy considered when setting the
            humidity. Default is 3.0.
        `resource_path (Optional[str])`: The resource path of the environmental chamber.
            If None, the hostname must be provided. Can't be used with hostname. Default
            is None.
        `resource_namager (Optional[pyvisa.ResourceManager])`: An optional PyVISA
            resource manager. If None, the default one is used. Default is None.
        `communication_timeout (Optional[int])`: The communication timeout in
            milliseconds. Default is 5000.
    """

    MONITOR_COMMAND_DELAY = 0.2
    """Delay in seconds when sending a command to the environmental chamber
       (program-related delay is 0.3)"""

    SETTING_COMMAND_DELAY = 0.5
    """Delay in seconds when sending a setting command to the environmental chamber
       (program-related delay is 1)"""

    LINE_TERMINATION = "\r\n"
    """The line termination character used by the environmental chamber"""

    TCP_PORT = 57732
    """The TCP port of the environmental chamber"""

    def __init__(
        self,
        hostname: Optional[str] = None,
        temperature_accuracy: Optional[float] = None,
        humidity_accuracy: Optional[float] = None,
        resource_path: Optional[str] = None,
        resource_manager: Optional[pyvisa.ResourceManager] = None,
        communication_timeout: Optional[int] = None,
    ):
        assert (hostname is None) or (resource_path is None)
        assert (hostname is not None) or (resource_path is not None)

        self.hostname = hostname
        """The IP address of the environmental chamber"""

        self.temperature_accuracy = temperature_accuracy or 0.5
        """The accuracy considered when setting the temperature"""

        self.humidity_accuracy = humidity_accuracy or 3.0
        """The accuracy considered when setting the humidity"""

        # we try to connect to the environmental chamber just to see if there is an
        # error
        self._resource_manager = resource_manager or pyvisa.ResourceManager()

        if resource_path is None:
            resource_path = f"TCPIP0::{self.hostname}"  # noqa E231
            resource_path += f"::{self.TCP_PORT}::SOCKET"  # noqa E231

        self.resource_path = resource_path
        """Resource path of the environmental chamber"""

        self._chamber = self._resource_manager.open_resource(self.resource_path)
        _LOGGER.debug(f"Connected to the environmental chamber at {self.resource_path}")

        self._chamber.write_termination = self.LINE_TERMINATION
        self._chamber.read_termination = self.LINE_TERMINATION
        self._chamber.timeout = communication_timeout or 5000

    def _target_temperature_reached(self) -> bool:
        """
        Checks if the current temperature is within the target temperature range.
        """
        temperature_status = self.get_temperature_status()
        current = temperature_status.current_temperature
        target = temperature_status.target_temperature

        _LOGGER.debug(
            f"Current temperature: {current}°C, Target temperature: {target} +-"
            f"{self.temperature_accuracy}°C"
        )
        return abs(current - target) <= self.temperature_accuracy

    def _target_humidity_reached(self) -> bool:
        """
        Checks if the current humidity is within the target humidity range. If the
        humidity control is disabled, it always returns True.
        """
        humidity_status = self.get_humidity_status()
        current = humidity_status.current_humidity
        target = humidity_status.target_humidity

        if target is None:
            _LOGGER.debug("Humidity control is disabled")
            return True

        _LOGGER.debug(
            f"Current humidity: {current}%, Target humidity: {target} +-"
            f"{self.humidity_accuracy}%"
        )
        return abs(current - target) <= self.humidity_accuracy

    def get_temperature_status(self) -> TemperatureStatus:
        """
        Gets the temperature status of the environmental chamber. This includes the
        current temperature, set temperature, upper limit, and lower limit.

        Raises:
            `MonitorError`: If an error occurred when getting the temperature status.
        """
        # send the request to the chamber
        response = self._chamber.query("TEMP?", delay=self.MONITOR_COMMAND_DELAY)

        # data format: [current temp, set temp, upper limit, lower limit]
        pattern = re.compile(
            r"(?P<current>\d+\.\d+)"
            r",(?P<target>\d+\.\d+)"
            r",(?P<upper>\d+\.\d+)"
            r",(?P<lower>\d+\.\d+)"
        )

        match = pattern.match(response)
        if match is None:
            _LOGGER.error("Failed to get the temperature status")
            _LOGGER.debug(f"Response: '{response}'")
            raise MonitorError("Failed to get the temperature status")

        # convert into float numbers
        temperature_status = TemperatureStatus(
            current_temperature=float(match["current"]),
            target_temperature=float(match["target"]),
            upper_limit=float(match["upper"]),
            lower_limit=float(match["lower"]),
        )

        return temperature_status

    def get_humidity_status(self) -> HumidityStatus:
        """
        Gets the humidity status of the environmental chamber. This includes the current
        humidity, set humidity, upper limit, and lower limit.

        Raises:
            `MonitorError`: If an error occurred when getting the humidity status.
        """
        # send the request to the chamber
        response = self._chamber.query("HUMI?", delay=self.MONITOR_COMMAND_DELAY)

        # data format: [current humi, set humi, upper limit, lower limit]
        pattern = re.compile(
            r"(?P<current>\d+)"
            r",(?P<target>OFF|\d+)"
            r",(?P<upper>\d+)"
            r",(?P<lower>\d+)"
        )

        match = pattern.match(response)
        if match is None:
            _LOGGER.error("Failed to get the temperature status")
            _LOGGER.debug(f"Response: '{response}'")
            raise MonitorError("Failed to get the humidity status")

        if match["target"] == "OFF":
            target_humidity = None
        else:
            target_humidity = float(match["target"])

        # convert into float numbers
        humidity_status = HumidityStatus(
            current_humidity=float(match["current"]),
            target_humidity=target_humidity,
            upper_limit=float(match["upper"]),
            lower_limit=float(match["lower"]),
        )

        return humidity_status

    def set_target_temperature(self, temperature: float):
        """
        Sets the target temperature of the environmental chamber.

        Args:
            `temperature`: The target temperature to set in Celsius.

        Raises:
            `ClimateChamberSettingError`: If an error occurred when setting the
                target temperature.
        """
        # sets the temp of the chamber, temperature
        _LOGGER.debug(f"Setting target temperature to {temperature}°C")
        response = self._chamber.query(
            f"TEMP, S{temperature:.1f}", delay=self.SETTING_COMMAND_DELAY  # noqa E231
        )

        # verify the response
        response_pattern = re.compile(r"OK:TEMP, S\d+.\d+")
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the target temperature")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the target temperature")

    def set_target_humidity(self, humidity: Optional[float] = None):
        """
        Sets the target humidity of the environmental chamber.

        Args:
            `humidity`: The target humidity to set in percentage. If None, the humidity
                control is disabled.

        Raises:
            `ClimateChamberSettingError`: If an error occurred when setting the
                target humidity.
        """
        if humidity is None:
            _LOGGER.debug("Disabling humidity control")
            response = self._chamber.query(
                "HUMI, SOFF", delay=self.SETTING_COMMAND_DELAY
            )
            response_pattern = re.compile(r"OK:HUMI, SOFF")
        else:
            # sets the humidity of the chamber, (float) humidity
            _LOGGER.debug(f"Setting target humidity to {humidity}%")
            response = self._chamber.query(
                f"HUMI, S{humidity}", delay=self.SETTING_COMMAND_DELAY
            )
            response_pattern = re.compile(r"OK:HUMI, S\d+.\d+")

        # verify the response
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the target humidity")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the target humidity")

    def close(self):
        """
        Closes the connection to the environmental chamber.
        """
        _LOGGER.debug("Closing the connection to the environmental chamber")
        self._chamber.close()

    def get_test_area_state(self) -> TestAreaState:
        """
        Get the chamber test area state.
        """
        response: str = self._chamber.query("MON?", delay=self.MONITOR_COMMAND_DELAY)

        # output data format: [temp, humid, op-state, num. of alarms]
        pattern = re.compile(
            r"(?P<temp>\d+\.\d+),(?P<humid>\d+),(?P<state>\w+),(?P<alarms>\d+)"
        )

        match = pattern.match(response)
        if match is None:
            _LOGGER.error("Failed to get the test area state")
            _LOGGER.debug(f"Response: '{response}'")
            raise MonitorError("Failed to get the test area state")

        test_area_state = TestAreaState(
            current_temperature=float(match["temp"]),
            current_humidity=float(match["humid"]),
            operation_state=OperationMode.from_str(match["state"]),
            number_of_alarms=int(match["alarms"]),
        )
        return test_area_state

    def set_temperature_limits(self, upper_limit: float, lower_limit: float):
        """
        Sets the upper and lower temperature limits for the chamber.

        Args:
            `upper_limit`: The temperature upper limit in Celsius.
            `lower_limit`: The temperature lower limit in Celsius.

        Raises:
            `ClimateChamberSettingError`: If an error occurred when setting the
                temperature limits.
        """
        _LOGGER.debug(
            f"Setting temperature limits to {upper_limit}°C and {lower_limit}°C"
        )
        response = self._chamber.query(f"TEMP, H{upper_limit: 0.1f}")

        response_pattern = re.compile(r"OK:TEMP, H \d+.\d+")
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the upper temperature limit")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the upper temperature limit")

        response = self._chamber.query(f"TEMP, L{lower_limit: 0.1f}")

        response_pattern = re.compile(r"OK:TEMP, L \d+.\d+")
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the lower temperature limit")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the lower temperature limit")

    def set_humidity_limits(self, upper_limit: float, lower_limit: float):
        """
        Sets the upper and lower humidity limits for the chamber

        Args:
            `upper_limit`: The humidity upper limit.
            `lower_limit`: The humidity lower limit.

        Raises:
            `ClimateChamberSettingError`: If an error occurred when setting the
                humidity limits.
        """
        _LOGGER.debug(f"Setting humidity limits to {upper_limit}% and {lower_limit}%")
        response = self._chamber.query("HUMI, H" + str(upper_limit))

        response_pattern = re.compile(r"OK:HUMI, H\d+")
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the upper humidity limit")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the upper humidity limit")

        response = self._chamber.query("HUMI, L" + str(lower_limit))

        response_pattern = re.compile(r"OK:HUMI, L\d+")
        if not response_pattern.match(response):
            _LOGGER.error("Failed to set the lower humidity limit")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the lower humidity limit")

    def get_mode(self) -> OperationMode:
        """
        Gets the operation mode of the environmental chamber.
        """
        response = self._chamber.query("MODE?", delay=self.MONITOR_COMMAND_DELAY)
        pattern = re.compile(r"(?P<mode>\w+)")
        match = pattern.match(response)

        if match is None:
            _LOGGER.error("Failed to get the operation mode")
            _LOGGER.debug(f"Response: '{response}'")
            raise MonitorError("Failed to get the operation mode")

        return OperationMode.from_str(response)

    def set_mode(self, mode: OperationMode):
        """
        Sets the operation mode of the environmental chamber.

        Args:
            `mode`: The operation mode to set.
        """
        # sets the mode of the chamber:
        _LOGGER.debug(f"Setting operation mode to {mode}")
        response = self._chamber.query(
            f"MODE, {mode}", delay=self.SETTING_COMMAND_DELAY
        )

        response_pattern = re.compile(r"OK:MODE, (?P<mode>\w+)")
        match = response_pattern.match(response)
        if match is None:
            _LOGGER.error("Failed to set the operation mode")
            _LOGGER.debug(f"Response: '{response}'")
            raise SettingError("Failed to set the operation mode")

        set_mode = OperationMode.from_str(match["mode"])
        if set_mode != mode:
            _LOGGER.error(
                f"Operation mode not set correctly"
                f" (current: {match['mode']}, expected: {mode})"
            )
            _LOGGER.debug(f"Response: '{response}'")
            _LOGGER.debug(f"Mode: '{match['mode']}'")
            raise SettingError("Failed to set the operation mode")

        return response

    def set_constant_condition(
        self,
        temperature: float,
        humidity: Optional[float] = None,
        stable_time=60.0,
        poll_interval=1.0,
    ):
        """
        Sets the environmental chamber to a constant temperature and humidity condition
        and waits until the setpoints are reached and stable.

        Args:
            `temperature`: The temperature to set in Celsius.
            `humidity`: The humidity to set in percentage. Default is None (humidity
                control is disabled)
            `stable_time`: The time in seconds to wait until the setpoints are stable.
                Default is 60.
            `poll_interval`: The time in seconds to wait between each check.
                Default is 1.
        """
        _LOGGER.debug(f"Setting constant condition {temperature}°C, {humidity}%")

        self.set_target_temperature(temperature)
        self.set_target_humidity(humidity)
        self.set_mode(OperationMode.CONSTANT)

        start_time = time.time()

        _LOGGER.debug("Waiting for the setpoints to be reached")

        while True:
            stable = (
                self._target_temperature_reached() and self._target_humidity_reached()
            )
            if not stable:
                _LOGGER.debug("Setpoints not reached yet")
                start_time = time.time()

            if time.time() - start_time >= stable_time:
                _LOGGER.debug("Setpoints reached and stable")
                break

            time.sleep(poll_interval)

    def get_heater_percentage(self) -> HeatersStatus:
        """
        Gets the output of the heaters
        """
        response = self._chamber.query("%?", delay=self.MONITOR_COMMAND_DELAY)

        pattern = re.compile(r"\d+,(?P<temp>\d+\.\d+),(?P<humid>\d+\.\d+)")
        match = pattern.match(response)

        if match is None:
            _LOGGER.error("Failed to get the heaters status")
            _LOGGER.debug(f"Response: '{response}'")
            raise MonitorError("Failed to get the heaters status")

        heaters = HeatersStatus(
            temperature_heater=float(match["temp"]),
            humidity_heater=float(match["humid"]),
        )

        return heaters

    def __del__(self):
        self.close()



class SettingError(Exception):
    """
    An error occurred when sending a setting command to the environmental chamber.
    """

    pass


class MonitorError(Exception):
    """
    An error occurred when sending a monitor command to the environmental chamber.
    """

    pass



import pytest

from espec_pr3j import EspecPr3j, OperationMode

TARGET_TEMPERATURE = 23.0
LOWER_TEMPERATURE = 20.0
UPPER_TEMPERATURE = 30.0

TARGET_HUMIDITY = 50.0
LOWER_HUMIDITY = 40.0
UPPER_HUMIDITY = 99.0


@pytest.fixture
def STABILITY_POLL_INTERVAL(hil):
    if hil:
        return 30

    return 0.001


@pytest.fixture
def STABILITY_TIME(hil):
    if hil:
        return 60 * 5

    return 0.01


def test_temperature_limits(environmental_chamber: EspecPr3j):
    environmental_chamber.set_temperature_limits(
        upper_limit=UPPER_TEMPERATURE, lower_limit=LOWER_TEMPERATURE
    )

    temperature = environmental_chamber.get_temperature_status()

    assert temperature.lower_limit == LOWER_TEMPERATURE
    assert temperature.upper_limit == UPPER_TEMPERATURE


def test_humidity_limits(environmental_chamber: EspecPr3j):
    environmental_chamber.set_humidity_limits(
        upper_limit=UPPER_HUMIDITY, lower_limit=LOWER_HUMIDITY
    )

    humidity = environmental_chamber.get_humidity_status()

    assert humidity.lower_limit == LOWER_HUMIDITY
    assert humidity.upper_limit == UPPER_HUMIDITY


@pytest.mark.skipif(
    "config.getvalue('hil')", reason="Not valid for hardware-in-the-loop"
)
def test_modes(environmental_chamber: EspecPr3j):
    environmental_chamber.set_mode(OperationMode.CONSTANT)
    assert environmental_chamber.get_mode() == OperationMode.CONSTANT

    environmental_chamber.set_mode(OperationMode.RUN)
    assert environmental_chamber.get_mode() == OperationMode.RUN

    environmental_chamber.set_mode(OperationMode.OFF)
    assert environmental_chamber.get_mode() == OperationMode.OFF

    environmental_chamber.set_mode(OperationMode.STANDBY)
    assert environmental_chamber.get_mode() == OperationMode.STANDBY


def test_constant_condition(
    environmental_chamber: EspecPr3j, STABILITY_TIME, STABILITY_POLL_INTERVAL
):
    environmental_chamber.set_constant_condition(
        temperature=TARGET_TEMPERATURE,
        humidity=TARGET_HUMIDITY,
        stable_time=STABILITY_TIME,
        poll_interval=STABILITY_POLL_INTERVAL,
    )

    temperature = environmental_chamber.get_temperature_status()
    humidity = environmental_chamber.get_humidity_status()

    assert (
        abs(temperature.current_temperature - TARGET_TEMPERATURE)
        < environmental_chamber.temperature_accuracy
    )
    assert (
        abs(humidity.current_humidity - TARGET_HUMIDITY)
        < environmental_chamber.humidity_accuracy
    )


def test_no_humidity_control(environmental_chamber: EspecPr3j):
    environmental_chamber.set_target_humidity(None)

    humidity = environmental_chamber.get_humidity_status()
    assert humidity.target_humidity is None

    environmental_chamber.set_target_humidity(TARGET_HUMIDITY)
    humidity = environmental_chamber.get_humidity_status()
    assert humidity.target_humidity == TARGET_HUMIDITY


def test_no_humidity_control_in_constant_condition(
    environmental_chamber: EspecPr3j, STABILITY_TIME, STABILITY_POLL_INTERVAL
):
    environmental_chamber.set_constant_condition(
        temperature=TARGET_TEMPERATURE,
        humidity=None,
        stable_time=STABILITY_TIME,
        poll_interval=STABILITY_POLL_INTERVAL,
    )

    temperature = environmental_chamber.get_temperature_status()
    humidity = environmental_chamber.get_humidity_status()

    assert (
        abs(temperature.current_temperature - TARGET_TEMPERATURE)
        < environmental_chamber.temperature_accuracy
    )
    assert humidity.target_humidity is None


def test_test_area_state(environmental_chamber: EspecPr3j):
    test_area = environmental_chamber.get_test_area_state()

    assert (
        abs(test_area.current_temperature - TARGET_TEMPERATURE)
        < environmental_chamber.temperature_accuracy
    )

    assert (
        abs(test_area.current_humidity - TARGET_HUMIDITY)
        < environmental_chamber.humidity_accuracy
    )

    assert test_area.operation_state == OperationMode.CONSTANT


def test_heaters(environmental_chamber: EspecPr3j):
    heaters_status = environmental_chamber.get_heater_percentage()

    assert heaters_status is not None
    assert 0.0 <= heaters_status.humidity_heater <= 100.0
    assert 0.0 <= heaters_status.temperature_heater <= 100.0



from .data_classes import (
    HeatersStatus,
    HumidityStatus,
    OperationMode,
    TemperatureStatus,
    TestAreaState,
)
from .espec_pr3j import EspecPr3j
from .exceptions import SettingError

__all__ = [
    "EspecPr3j",
    "HumidityStatus",
    "TemperatureStatus",
    "SettingError",
    "HeatersStatus",
    "OperationMode",
    "TestAreaState",
]



"""
    Setup file for espec_pr3j
    Use setup.cfg to configure your project.
"""
from setuptools import setup

if __name__ == "__main__":
    try:
        setup(use_scm_version={"version_scheme": "no-guess-dev"})
    except:  # noqa
        print(
            "\n\nAn error occurred while building the project, "
            "please ensure you have the most updated version of setuptools, "
            "setuptools_scm and wheel with:\n"
            "   pip install -U setuptools setuptools_scm wheel\n\n"
        )
        raise
